%   File: 'run_sim.m'
%
%   Author(s):  Rogerio Richa
%   Created on: 2011
% 
%   (C) Copyright 2006-2011 Johns Hopkins University (JHU), All Rights
%   Reserved.
% 
% --- begin cisst license - do not edit ---
% 
% This software is provided "as is" under an open source license, with
% no warranty.  The complete license can be found in license.txt and
% http://www.cisst.org/cisst/license.txt.
% 
% --- end cisst license ---

clear all
close all
clc

input_parameters;

% Initialization for first image
nb_image = nb_first_image;
loads_image;

% Selects reference image (Template)
size_template_x = size_template_x/2; % in the rest of the code, this variable will represent only half the template size ... 
size_template_y = size_template_y/2;  

Template = double(ICur_gray(pos(2)-size_template_y+1:pos(2)+size_template_y, pos(1)-size_template_x+1: pos(1)+size_template_x));

figure(4)
imshow(uint8(Template))
title('Reference image')

% Initializing tracking parameters
H = [ 1 0 pos(1)-size_template_x; 0 1 pos(2)-size_template_y; 0 0 1];
    
% Initializing SCV parameters
size_bin = 256 / n_bins;
Nx = numel(Template);
expected_grayscale = 1:n_bins+1;

% Initializing expected image T_hat
Template = round(Template./size_bin);
[Ix_t, Iy_t] = gradient(Template);
Expected = Template(:);

% Runs on dataset
for nb_image = nb_first_image +  1 : nb_last_image
        
    % Loads image
    loads_image
    
    % Tracking
    [H, Expected, Ix_t, Iy_t, Warped, p_joint] = tracking_scv(ICur_gray, H, Expected, Template(:), n_bins, size_bin, Nx, size_template_x, size_template_y, Ix_t, Iy_t, epsilon, maxIters);
   
    % Display
    figure(1) ; clf ;
    imshow(uint8(ICur)); axis image ;
    hold on
    
    T_pos = H*[0 2*size_template_x 2*size_template_x 0 0; 0 0 2*size_template_y  2*size_template_y 0; ones(1,5)];
    T_pos(1,:) = T_pos(1,:)./T_pos(3,:);
    T_pos(2,:) = T_pos(2,:)./T_pos(3,:);
    
    plot(T_pos(1,:), T_pos(2,:),'g-','LineWidth',2,'MarkerSize',6)
    drawnow
    
    figure(2)
    imagesc(p_joint)
    title('Current joint intensity distribution between Figs. 3 and 4')
    
    figure(3)
    imshow(uint8(size_bin.*Warped));
    title('Current back-warped image');
    
    % Saving results into a file
    % saveas(figure(1), strcat( './sauvegarde/', 'histo_', number, '.fig' ))
    % print(figure(2), strcat( './sauvegarde/', 'warped_', number, '.png' ), '-dpng', '-r150');
    % print(figure(3), strcat( './sauvegarde/', 'cross_', number, '.png' ), '-dpng', '-r150');
end


